# rspec-power_assert
require 'timesteps'
require 'rspec-power_assert'

unless defined? CALENDAR_NAMES
  CALENDAR_NAMES = [
    "standard",
    "proleptic_gregorian",
    "proleptic_julian",
    "noleap",
    "allleap",
    "360_day"
  ]
end

describe "TimeStep.new" do

  example 'origin' do 

    spec = "1 days since 2001-02-03 04:05:06 +07:00"
    tests = [
      [spec, "standard", DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::ITALY)],
      [spec, "gregorian", DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::ITALY)],
      [spec, "proleptic_gregorian", DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::GREGORIAN)],
      [spec, "proleptic_julian", DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::JULIAN)],
      [spec, "noleap",  DateTime::NoLeap.new(2001,2,3,4,5,6,7.quo(24))],
      [spec, "365_day", DateTime::NoLeap.new(2001,2,3,4,5,6,7.quo(24))],
      [spec, "allleap", DateTime::AllLeap.new(2001,2,3,4,5,6,7.quo(24))],
      [spec, "366_day", DateTime::AllLeap.new(2001,2,3,4,5,6,7.quo(24))],
      [spec, "360_day", DateTime::Fixed360Day.new(2001,2,3,4,5,6,7.quo(24))],
    ]

    tests.each do |spec, calendar, date|

      ts = TimeStep.new(spec, calendar: calendar)

      is_asserted_by { ts.origin == date }

    end
    
  end

  example 'interval' do

    tests = [
      ["year since 2001-02-03 04:05:06 +07:00",   1, :years],
      ["month since 2001-02-03 04:05:06 +07:00",  1, :months],
      ["day since 2001-02-03 04:05:06 +07:00",    1, :days],
      ["hour since 2001-02-03 04:05:06 +07:00",   1, :hours],
      ["minute since 2001-02-03 04:05:06 +07:00", 1, :minutes],
      ["second since 2001-02-03 04:05:06 +07:00", 1, :seconds],

      ["d since 2001-02-03 04:05:06 +07:00",    1, :days],
      ["hr since 2001-02-03 04:05:06 +07:00",   1, :hours],
      ["h since 2001-02-03 04:05:06 +07:00",   1, :hours],
      ["min since 2001-02-03 04:05:06 +07:00", 1, :minutes],
      ["sec since 2001-02-03 04:05:06 +07:00", 1, :seconds],
      ["s since 2001-02-03 04:05:06 +07:00", 1, :seconds],

      ["3 years since 2001-02-03 04:05:06 +07:00",   3, :years],
      ["3 months since 2001-02-03 04:05:06 +07:00",  3, :months],
      ["3 days since 2001-02-03 04:05:06 +07:00",    3, :days],
      ["3 hours since 2001-02-03 04:05:06 +07:00",   3, :hours],
      ["3 minutes since 2001-02-03 04:05:06 +07:00", 3, :minutes],
      ["3 seconds since 2001-02-03 04:05:06 +07:00", 3, :seconds],

      ["3 years since    2001-02-03 04:05:06 +07:00",   3, :years],
      ["3 months since   2001-02-03 04:05:06 +07:00",  3, :months],
      ["3 days since     2001-02-03 04:05:06 +07:00",    3, :days],
      ["3 hours since    2001-02-03 04:05:06 +07:00",   3, :hours],
      ["3 minutes since  2001-02-03 04:05:06 +07:00", 3, :minutes],
      ["3 seconds since  2001-02-03 04:05:06 +07:00", 3, :seconds],

      ["3years since 2001-02-03 04:05:06 +07:00",   3, :years],
      ["3months since 2001-02-03 04:05:06 +07:00",  3, :months],
      ["3days since 2001-02-03 04:05:06 +07:00",    3, :days],
      ["3hours since 2001-02-03 04:05:06 +07:00",   3, :hours],
      ["3minutes since 2001-02-03 04:05:06 +07:00", 3, :minutes],
      ["3seconds since 2001-02-03 04:05:06 +07:00", 3, :seconds],

      ["60 years since 2001-02-03 04:05:06 +07:00",   60, :years],
      ["60 months since 2001-02-03 04:05:06 +07:00",  60, :months],
      ["60 days since 2001-02-03 04:05:06 +07:00",    60, :days],
      ["60 hours since 2001-02-03 04:05:06 +07:00",   60, :hours],
      ["60 minutes since 2001-02-03 04:05:06 +07:00", 60, :minutes],
      ["60 seconds since 2001-02-03 04:05:06 +07:00", 60, :seconds],

      ["-3 years since 2001-02-03 04:05:06 +07:00",   -3, :years],
      ["-3 months since 2001-02-03 04:05:06 +07:00",  -3, :months],
      ["-3 days since 2001-02-03 04:05:06 +07:00",    -3, :days],
      ["-3 hours since 2001-02-03 04:05:06 +07:00",   -3, :hours],
      ["-3 minutes since 2001-02-03 04:05:06 +07:00", -3, :minutes],
      ["-3 seconds since 2001-02-03 04:05:06 +07:00", -3, :seconds],

      ["3.5 days since 2001-02-03 04:05:06 +07:00",    3.5, :days],
      ["3.5 hours since 2001-02-03 04:05:06 +07:00",   3.5, :hours],
      ["3.5 minutes since 2001-02-03 04:05:06 +07:00", 3.5, :minutes],
      ["3.5 seconds since 2001-02-03 04:05:06 +07:00", 3.5, :seconds],

      ["3.5e2 days since 2001-02-03 04:05:06 +07:00",    3.5e2, :days],
      ["3.5e2 hours since 2001-02-03 04:05:06 +07:00",   3.5e2, :hours],
      ["3.5e2 minutes since 2001-02-03 04:05:06 +07:00", 3.5e2, :minutes],
      ["3.5e2 seconds since 2001-02-03 04:05:06 +07:00", 3.5e2, :seconds],

      ["3.5e21 days since 2001-02-03 04:05:06 +07:00",    3.5e21, :days],
      ["3.5e21 hours since 2001-02-03 04:05:06 +07:00",   3.5e21, :hours],
      ["3.5e21 minutes since 2001-02-03 04:05:06 +07:00", 3.5e21, :minutes],
      ["3.5e21 seconds since 2001-02-03 04:05:06 +07:00", 3.5e21, :seconds],

      ["3.5e-2 days since 2001-02-03 04:05:06 +07:00",    3.5e-2, :days],
      ["3.5e-2 hours since 2001-02-03 04:05:06 +07:00",   3.5e-2, :hours],
      ["3.5e-2 minutes since 2001-02-03 04:05:06 +07:00", 3.5e-2, :minutes],
      ["3.5e-2 seconds since 2001-02-03 04:05:06 +07:00", 3.5e-2, :seconds],
    ]
    
    tests.each do |spec, numeric, symbol|
      ts = TimeStep.new(spec)

      is_asserted_by { ts.numeric == numeric }
      is_asserted_by { ts.symbol  == symbol }
    end
    
  end
  
  example 'since option with format' do
    
    time = "20010203_040506.dat"
    template = "%Y%m%d_%H%M%S.dat"

    ts = TimeStep.new("3 days", since: time, format: template)
    
    is_asserted_by { ts.origin.strftime(template) == time }
    is_asserted_by { ts.numeric == 3 }
    is_asserted_by { ts.symbol  == :days }

    ts = TimeStep.new("3 days", since: time, format: template, calendar: "noleap")
    
    is_asserted_by { ts.origin.strftime(template) == time }
    is_asserted_by { ts.numeric == 3 }
    is_asserted_by { ts.symbol  == :days }
    
  end

  example 'since option' do
    
    time = DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::ITALY)
    ts = TimeStep.new("3 days", since: time)
    
    is_asserted_by { ts.origin == time }
    is_asserted_by { ts.numeric == 3 }
    is_asserted_by { ts.symbol  == :days }

    time = DateTime::NoLeap.new(2001,2,3,4,5,6,7.quo(24))
    ts = TimeStep.new("3 days", since: time, calendar: "noleap")
    
    is_asserted_by { ts.origin == time }
    is_asserted_by { ts.numeric == 3 }
    is_asserted_by { ts.symbol  == :days }
    
  end

  example 'since option with calendar mismatch' do
    time = DateTime.new(2001,2,3,4,5,6,7.quo(24),Date::GREGORIAN)
    expect { TimeStep.new("3 days", since: time) }.to raise_error(RuntimeError)
    expect { TimeStep.new("3 days", since: time, calendar: "noleap") }.to raise_error(RuntimeError)

    time = DateTime::NoLeap.new(2001,2,3,4,5,6,7.quo(24))
    expect { TimeStep.new("3 days", since: time) }.to raise_error(RuntimeError)
    expect { TimeStep.new("3 days", since: time, calendar: "allleap") }.to raise_error(RuntimeError)
  end

end

describe "TimeStep#time_at" do


  example 'invalid argument' do
    
    ts = TimeStep.new("1 seconds since 2001-02-03 04:05:06 +07:00")
    
    expect { ts.time_at("string") }.to raise_error(ArgumentError)
    
  end


  example 'seconds' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 seconds since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.time_at(0) == ts.origin }
      is_asserted_by { ts.time_at(1) == ts.origin + 1.quo(86400) }
      is_asserted_by { ts.time_at(1000000) == ts.origin + 1000000.quo(86400) }
      is_asserted_by { ts.time_at(-1) == ts.origin - 1.quo(86400) }
      is_asserted_by { ts.time_at(-1000000) == ts.origin - 1000000.quo(86400) }

    end
    
  end

  example 'days' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 days since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.time_at(0) == ts.origin }
      is_asserted_by { ts.time_at(1) == ts.origin.next_day(1) }
      is_asserted_by { ts.time_at(366) == ts.origin.next_day(366) }
      is_asserted_by { ts.time_at(-1) == ts.origin.prev_day(1) }
      is_asserted_by { ts.time_at(-366) == ts.origin.prev_day(366) }

      is_asserted_by { ts.time_at(1.5) == (ts.origin + 1.5) }
      is_asserted_by { ts.time_at(-1.5) == (ts.origin - 1.5) }

    end
    
  end

  example 'months' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 months since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.time_at(0) == ts.origin }
      is_asserted_by { ts.time_at(1) == ts.origin.next_month(1) }
      is_asserted_by { ts.time_at(366) == ts.origin.next_month(366) }
      is_asserted_by { ts.time_at(-1) == ts.origin.prev_month(1) }
      is_asserted_by { ts.time_at(-366) == ts.origin.prev_month(366) }

      # months timestep don't permit fractional index
      expect { ts.time_at(1.5) }.to raise_error(ArgumentError)

    end
    
  end

  example 'years' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 years since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.time_at(0) == ts.origin }
      is_asserted_by { ts.time_at(1) == ts.origin.next_year(1) }
      is_asserted_by { ts.time_at(366) == ts.origin.next_year(366) }
      is_asserted_by { ts.time_at(-1) == ts.origin.prev_year(1) }
      is_asserted_by { ts.time_at(-366) == ts.origin.prev_year(366) }

      # years timestep don't permit fractional index
      expect { ts.time_at(1.5) }.to raise_error(ArgumentError)

    end
    
  end

end

describe "TimeStep#index_at" do

  example 'seconds' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 seconds since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.index_at( ts.origin ) == 0 }
      is_asserted_by { ts.index_at( ts.origin + 1.quo(86400) ) == 1 }
      is_asserted_by { ts.index_at( ts.origin + 100.quo(86400) ) == 100 }
      is_asserted_by { ts.index_at( ts.origin + 0.5.to_r.quo(86400) ) == 0.5 }
      is_asserted_by { ts.index_at( ts.origin - 0.5.to_r.quo(86400) ) == -0.5 }

    end
    
  end

  example 'days' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 days since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.index_at( ts.origin ) == 0 }
      is_asserted_by { ts.index_at( ts.origin + 1 ) == 1 }
      is_asserted_by { ts.index_at( ts.origin + 100 ) == 100 }
      is_asserted_by { ts.index_at( ts.origin + 0.5 ) == 0.5 }
      is_asserted_by { ts.index_at( ts.origin - 0.5 ) == -0.5 }

    end
    
  end

  example 'months' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 months since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.index_at( ts.origin ) == 0 }
      is_asserted_by { ts.index_at( ts.origin.next_month(1) ) == 1 }
      is_asserted_by { ts.index_at( ts.origin.next_month(100) ) == 100 }
      is_asserted_by { ts.index_at( ts.origin.prev_month(1) ) == -1 }
      is_asserted_by { ts.index_at( ts.origin.prev_month(100) ) == -100 }

    end
    
  end

  example 'years' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 years since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.index_at( ts.origin ) == 0 }
      is_asserted_by { ts.index_at( ts.origin.next_year(1) ) == 1 }
      is_asserted_by { ts.index_at( ts.origin.next_year(100) ) == 100 }
      is_asserted_by { ts.index_at( ts.origin.prev_year(1) ) == -1 }
      is_asserted_by { ts.index_at( ts.origin.prev_year(100) ) == -100 }
      is_asserted_by { ts.index_at( ts.origin.next_year(1) - 1 ).floor == 0 }
      is_asserted_by { ts.index_at( ts.origin.next_year(100) - 1 ).floor == 99 }
      is_asserted_by { ts.index_at( ts.origin.prev_year(1) + 1 ).floor == -1 }
      is_asserted_by { ts.index_at( ts.origin.prev_year(100) + 1 ).floor == -100 }

    end
    
  end

  example 'unix' do 
    
    unixtime = TimeStep.new("seconds since 1970-01-01 00:00:00")
    time = DateTime.parse("2001-01-01 00:00:00 +09:00")

    is_asserted_by { unixtime.index_at(time) == 978274800 }
  
  end

end

describe "TimeStep#duration_at" do

  example 'seconds' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 seconds since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.duration_at( 1 ) == 1.quo(86400) }
      is_asserted_by { ts.duration_at( 100 ) == 100.quo(86400) }
      is_asserted_by { ts.duration_at( 0.5 ) ==  0.5.to_r.quo(86400) }
      is_asserted_by { ts.duration_at( -0.5 ) == -0.5.to_r.quo(86400) }

    end
    
  end

  example 'duration' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 days since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.duration_at( 1 ) == 1 }
      is_asserted_by { ts.duration_at( 100 ) == 100 }
      is_asserted_by { ts.duration_at( 0.5 ) == 0.5 }
      is_asserted_by { ts.duration_at( -0.5 ) == -0.5 }

    end
    
  end

  example 'months' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 months since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.duration_at( 1 ) == ts.origin.next_month - ts.origin }
      is_asserted_by { ts.duration_at( 100 ) == ts.origin.next_month(100) - ts.origin }
      is_asserted_by { ts.duration_at( -1 ) == ts.origin.prev_month - ts.origin }
      is_asserted_by { ts.duration_at( -100 ) == ts.origin.prev_month(100) - ts.origin }

    end
    
  end

  example 'years' do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 years since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.duration_at( 1 ) == ts.origin.next_year - ts.origin }
      is_asserted_by { ts.duration_at( 100 ) == ts.origin.next_year(100) - ts.origin }
      is_asserted_by { ts.duration_at( -1 ) == ts.origin.prev_year - ts.origin }
      is_asserted_by { ts.duration_at( -100 ) == ts.origin.prev_year(100) - ts.origin }

    end
    
  end

end

describe "TimeStep#shift_origin" do
  
  example do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 years since 2001-02-03 04:05:06 +07:00", calendar: calendar)

      is_asserted_by { ts.shift_origin(1).origin == ts.time_at(1) }

    end

  end

end

describe "TimeStep#new_origin" do
  
  example do 

    CALENDAR_NAMES.each do |calendar|

      ts = TimeStep.new("1 years since 2001-02-03 04:05:06 +07:00", calendar: calendar)
      d = ts.time_at(1)

      is_asserted_by { ts.new_origin(d).origin == d }

    end

  end

end

