# rspec-power_assert
require 'timesteps'
require 'rspec-power_assert'

describe "TimeStep::Pair.new" do

  example "spec" do 
    
    spec1 = "1 days since 2001-02-03 04:05:06 +07:00"
    spec2 = "1 hours since 2001-02-03 04:05:06 +07:00"

    pair = TimeStep::Pair.new(spec1, spec2)

    ts1 = TimeStep.new(spec1)
    ts2 = TimeStep.new(spec2)

    is_asserted_by { pair.from == ts1 }
    is_asserted_by { pair.to == ts2 }
    is_asserted_by { pair.difference == 0 }

  end

end

describe "TimeStep::Pair#forward" do

  example "different hours" do 
    
    spec1 = "1 hours since 2001-02-03 04:05:06 +07:00"
    spec2 = "3 hours since 2001-02-02 04:05:06 +07:00"

    pair = TimeStep::Pair.new(spec1, spec2)

    is_asserted_by { pair.forward(0)  == 24.quo(3) }
    is_asserted_by { pair.forward(1)  == 25.quo(3) }
    is_asserted_by { pair.forward(24) == 48.quo(3) }
    is_asserted_by { pair.forward(1.5) == (51.quo(2)).quo(3) }

  end

  example "different days" do 
    
    spec1 = "3 days since 2001-02-03 04:05:06 +07:00"
    spec2 = "1 days since 2001-02-02 04:05:06 +07:00"

    pair = TimeStep::Pair.new(spec1, spec2)

    is_asserted_by { pair.forward(0)  == 1 }
    is_asserted_by { pair.forward(1)  == 4 }
    is_asserted_by { pair.forward(10)  == 31 }
    is_asserted_by { pair.forward(1.5) == 11.quo(2) }

  end

  example "different months" do 
    
    spec1 = "3 months since 2001-02-03 04:05:06 +07:00"
    spec2 = "1 months since 2001-01-03 04:05:06 +07:00"

    pair = TimeStep::Pair.new(spec1, spec2)

    is_asserted_by { pair.forward(0) == 1 }
    is_asserted_by { pair.forward(1) == 4 }
    is_asserted_by { pair.forward(10) == 31 }

    expect { pair.forward(1.5) }.to raise_error(ArgumentError)
  end

  example "different years" do 
    
    spec1 = "3 years since 2001-02-03 04:05:06 +07:00"
    spec2 = "1 years since 2000-02-03 04:05:06 +07:00"

    pair = TimeStep::Pair.new(spec1, spec2)

    is_asserted_by { pair.forward(0) == 1 }
    is_asserted_by { pair.forward(1) == 4 }
    is_asserted_by { pair.forward(10) == 31 }

    expect { pair.forward(1.5) }.to raise_error(ArgumentError)

  end

  example "different offset" do 
    
    ts   = TimeStep.new("hour since 2001-01-01 00:00:00 +09:00")
    unix = TimeStep.new("seconds since 1970-01-01 00:00:00")

    pair = TimeStep::Pair.new(ts, unix)

    is_asserted_by { pair.forward(0) == 978274800 }
    is_asserted_by { pair.forward_time(0) == DateTime.new(2000,12,31,15,0,0,0) }
    
  end


end

